@@ -21,6 +21,69 @@
 # Author(s): Johan Dahlin <jdahlin@async.com.br>
 #            Ronaldo Maia <romaia@async.com.br>
 #
+#
+# Design notes:
+#
+#   When inserting new text, supose, the entry, at some time is like this,
+#   ahd the user presses '0', for instance:
+#   --------------------------------
+#   | ( 1 2 )   3 4 5   - 6 7 8 9  |
+#   --------------------------------
+#              ^ ^     ^
+#              S P     E
+#
+#   S - start of the field (start)
+#   E - end of the field (end)
+#   P - pos - where the new text is being inserted. (pos)
+#
+#   So, the new text will be:
+#
+#     the old text, from 0 until P
+#   + the new text
+#   + the old text, from P until the end of the field, shifted to the
+#     right
+#   + the old text, from the end of the field, to the end of the text.
+#
+#   After inserting, the text will be this:
+#   --------------------------------
+#   | ( 1 2 )   3 0 4 5 - 6 7 8 9  |
+#   --------------------------------
+#              ^   ^   ^
+#              S   P   E
+#
+#
+#   When deleting some text, supose, the entry, at some time is like this:
+#   --------------------------------
+#   | ( 1 2 )   3 4 5 6 - 7 8 9 0  |
+#   --------------------------------
+#              ^ ^ ^   ^
+#              S s e   E
+#
+#   S - start of the field (_start)
+#   E - end of the field (_end)
+#   s - start of the text being deleted (start)
+#   e - end of the text being deleted (end)
+#
+#   end - start -> the number of characters being deleted.
+#
+#   So, the new text will be:
+#
+#     the old text, from 0 until the start of the text being deleted.
+#   + the old text, from the start of where the text is being deleted, until
+#     the end of the field, shifted to the left, end-start positions
+#   + the old text, from the end of the field, to the end of the text.
+#
+#   So, after the text is deleted, the entry will look like this:
+#
+#   --------------------------------
+#   | ( 1 2 )   3 5 6   - 7 8 9 0  |
+#   --------------------------------
+#                ^
+#                P
+#
+#   P = the position of the cursor after the deletion, witch is equal to
+#   start (s at the previous illustration)
+
 
 """
 An enchanced version of GtkEntry that supports icons and masks
@@ -40,7 +103,7 @@ from kiwi.ui.entrycompletion import KiwiEntryCompletion
 from kiwi.utils import PropertyObject, gsignal, gproperty, type_register
 
 if not environ.epydoc:
-    HAVE_2_6 = gtk.pygtk_version[:2] == (2, 6)
+    HAVE_2_6 = gtk.pygtk_version[:2] <= (2, 6)
 else:
     HAVE_2_6 = True
 
@@ -112,9 +175,13 @@ class KiwiEntry(PropertyObject, gtk.Entry):
         self.connect('focus', self._on_focus)
         self.connect('focus-out-event', self._on_focus_out_event)
         self.connect('move-cursor', self._on_move_cursor)
-        self.connect('button-press-event', self._on_button_press_event)
-        self.connect('notify::cursor-position',
-                     self._on_notify_cursor_position)
+
+        # Ideally, this should be connected to notify::cursor-position, but
+        # there seems to be a bug in gtk that the notification is not emited
+        # when it should.
+        # TODO: investigate that and report a bug.
+        self.connect('notify::selection-bound',
+                     self._on_notify_selection_bound)
 
         self._block_changed = False
 
@@ -214,7 +281,7 @@ class KiwiEntry(PropertyObject, gtk.Entry):
           - 'a' alphanumeric, honors the locale
           - 'A' alphanumeric, honors the locale
 
-        This is similar to MaskedTextBox: 
+        This is similar to MaskedTextBox:
         U{http://msdn2.microsoft.com/en-us/library/system.windows.forms.maskedtextbox.mask(VS.80).aspx}
 
         Example mask for a ISO-8601 date
@@ -597,9 +664,11 @@ class KiwiEntry(PropertyObject, gtk.Entry):
         model = completion.get_model()
         if not len(model):
             return
-
-        content = model[iter][COL_TEXT].lower()
-        return key.lower() in content
+        raw_content = model[iter][COL_TEXT]
+        if raw_content is not None:
+            return key.lower() in raw_content.lower()
+        else:
+            return False
 
     def _on_completion__match_selected(self, completion, model, iter):
         if not len(model):
@@ -671,34 +740,6 @@ class KiwiEntry(PropertyObject, gtk.Entry):
 
         return None
 
-#   When inserting new text, supose, the entry, at some time is like this,
-#   ahd the user presses '0', for instance:
-#   --------------------------------
-#   | ( 1 2 )   3 4 5   - 6 7 8 9  |
-#   --------------------------------
-#              ^ ^     ^
-#              S P     E
-#
-#   S - start of the field (start)
-#   E - end of the field (end)
-#   P - pos - where the new text is being inserted. (pos)
-#
-#   So, the new text will be:
-#
-#     the old text, from 0 until P
-#   + the new text
-#   + the old text, from P until the end of the field, shifted to the
-#     right
-#   + the old text, from the end of the field, to the end of the text.
-#
-#   After inserting, the text will be this:
-#   --------------------------------
-#   | ( 1 2 )   3 0 4 5 - 6 7 8 9  |
-#   --------------------------------
-#              ^   ^   ^
-#              S   P   E
-#
-
     def _insert_at_pos(self, text, new, pos):
         """
         Inserts the character at the give position in text. Note that the
@@ -760,38 +801,6 @@ class KiwiEntry(PropertyObject, gtk.Entry):
 
         self._really_insert_text(text, 0)
 
-#   When deleting some text, supose, the entry, at some time is like this:
-#   --------------------------------
-#   | ( 1 2 )   3 4 5 6 - 7 8 9 0  |
-#   --------------------------------
-#              ^ ^ ^   ^
-#              S s e   E
-#
-#   S - start of the field (_start)
-#   E - end of the field (_end)
-#   s - start of the text being deleted (start)
-#   e - end of the text being deleted (end)
-#
-#   end - start -> the number of characters being deleted.
-#
-#   So, the new text will be:
-#
-#     the old text, from 0 until the start of the text being deleted.
-#   + the old text, from the start of where the text is being deleted, until
-#     the end of the field, shifted to the left, end-start positions
-#   + the old text, from the end of the field, to the end of the text.
-#
-#   So, after the text is deleted, the entry will look like this:
-#
-#   --------------------------------
-#   | ( 1 2 )   3 5 6   - 7 8 9 0  |
-#   --------------------------------
-#                ^
-#                P
-#
-#   P = the position of the cursor after the deletion, witch is equal to
-#   start (s at the previous ilustration)
-
     def _on_delete_text(self, editable, start, end):
         if not self._mask or self._block_delete:
             return
@@ -806,6 +815,9 @@ class KiwiEntry(PropertyObject, gtk.Entry):
             self._on_delete_text(editable, start-1, start)
             return
 
+        # we just tried to delete, stop the selection.
+        self._selecting = False
+
         field = self._get_field_at_pos(end-1)
         # Outside a field. Cannot delete.
         if field is None:
@@ -843,10 +855,8 @@ class KiwiEntry(PropertyObject, gtk.Entry):
 
         # Position the cursor on the right place.
         self.set_position(new_pos)
-
-        if self.is_empty():
-            pos = self.get_field_pos(0)
-            self.set_position(pos)
+        if pos == new_pos:
+            self._handle_position_change()
 
     def _after_grab_focus(self, widget):
         # The text is selectet in grab-focus, so this needs to be done after
@@ -887,7 +897,7 @@ class KiwiEntry(PropertyObject, gtk.Entry):
 
         return True
 
-    def _on_notify_cursor_position(self, widget, pspec):
+    def _on_notify_selection_bound(self, widget, pspec):
         if not self._mask:
             return
 
@@ -897,25 +907,28 @@ class KiwiEntry(PropertyObject, gtk.Entry):
         if self._selecting:
             return
 
+        self._handle_position_change()
+
+    def _handle_position_change(self):
         pos = self.get_position()
         field = self._get_field_at_pos(pos)
 
-        if pos == 0:
-            self.set_position(self.get_field_pos(0))
-            return
-
-        text = self.get_text()
-        field = self._get_field_at_pos(pos)
-
         # Humm, the pos is not inside any field. Get the next pos inside
         # some field, depending on the direction that the cursor is
         # moving
         diff = pos - self._pos
+        # but move only one position at a time.
+        if diff:
+            diff /= abs(diff)
+
         _field = field
-        while _field is None and (len(text) > pos > 0) and diff:
-            pos += diff
-            _field = self._get_field_at_pos(pos)
-            self._pos = pos
+        if diff:
+            while _field is None and pos >= 0:
+                pos += diff
+                _field = self._get_field_at_pos(pos)
+                self._pos = pos
+            if pos < 0:
+                self._pos = self.get_field_pos(0)
 
         if field is None:
             self.set_position(self._pos)
@@ -936,12 +949,6 @@ class KiwiEntry(PropertyObject, gtk.Entry):
     def _on_move_cursor(self, entry, step, count, extend_selection):
         self._selecting = extend_selection
 
-    def _on_button_press_event(self, entry, event ):
-        if event.type == gtk.gdk.BUTTON_PRESS and event.button == 1:
-            self._selecting = True
-        elif event.type == gtk.gdk.BUTTON_RELEASE and event.button == 1:
-            self._selecting = True
-
     # IconEntry
 
     def set_tooltip(self, text):
