@@ -449,21 +449,21 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
 
     Signals
     =======
-      - B{row-activated} (object):
+      - B{row-activated} (list, object):
         - Emitted when a row is "activated", eg double clicked or pressing
           enter. See the GtkTreeView documentation for more information
-      - B{selection-changed} (object):
+      - B{selection-changed} (list, object):
         - Emitted when the selection changes for the ObjectList
           enter. See the documentation on GtkTreeSelection::changed
           for more information
-      - B{double-click} (object):
+      - B{double-click} (list, object):
         - Emitted when a row is double-clicked, mostly you want to use
           the row-activated signal instead to be able catch keyboard events.
-      - B{right-click} (object):
+      - B{right-click} (list, object):
         - Emitted when a row is clicked with the right mouse button.
-      - B{cell-edited} (object, attribute):
+      - B{cell-edited} (list, object, attribute):
         - Emitted when a cell is edited.
-      - B{has-rows} (bool):
+      - B{has-rows} (list, bool):
         - Emitted when the objectlist goes from an empty to a non-empty
           state or vice verse.
 
@@ -1691,12 +1691,23 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
 type_register(ObjectList)
 
 class ObjectTree(ObjectList):
+    """
+    Signals
+    =======
+      - B{row-expanded} (list, object):
+        - Emitted when a row is "expanded", eg the littler arrow to the left
+          is opened. See the GtkTreeView documentation for more information.
+    """
     __gtype_name__ = 'ObjectTree'
+
+    gsignal('row-expanded', object)
+
     def __init__(self, columns=[], objects=None, mode=gtk.SELECTION_BROWSE,
                  sortable=False, model=None):
         if not model:
             model = gtk.TreeStore(object)
         ObjectList.__init__(self, columns, objects, mode, sortable, model)
+        self.get_treeview().connect('row-expanded', self._on_treeview__row_expanded)
 
     def _append_internal(self, parent, instance, select, prepend):
         iters = self._iters
@@ -1775,6 +1786,9 @@ class ObjectTree(ObjectList):
         self.get_treeview().collapse_row(
             self._model[treeiter].path)
 
+    def _on_treeview__row_expanded(self, treeview, treeiter, treepath):
+        self.emit('row-expanded', self.get_model()[treeiter][COL_MODEL])
+
 type_register(ObjectTree)
 
 class ListLabel(gtk.HBox):
